@@ -1,86 +1,118 @@
 """Functions to read and parse osef files/streams."""
 import socket
-import sys
+import time
 import traceback
 from collections import namedtuple, deque
 from itertools import islice
 from struct import Struct, error
-from typing import Any, Iterable, Optional, Tuple, BinaryIO, Union, Iterator
+from typing import Any, Iterable, Optional, Tuple, Iterator
 from urllib.parse import urlparse
+import logging
 
 from osef import types
 
 
 # -- Public functions --
-
-
-def open_osef(path: str) -> Union[BinaryIO, socket.socket, None]:
-    """Open file path or tcp socket
+class OsefStream:
+    """Context manager class to open file path or tcp socket, then read its values.
 
     :param path: path to osef file or TCP stream if path has form *tcp://hostname:port*
-    :return: opened osef file as a binaryIO or a socket (None if file/stream could not be opened)
+    The sever may close the socket if client is too late.
     """
-    parsed = urlparse(path)
-    if path == "-":
-        return sys.stdin.buffer
 
-    if parsed.scheme == "tcp":
+    def __init__(self, path: str):
+        self._path = path
+        self._parsed_path = urlparse(self._path)
+        self._io_stream = None
+        self.is_tcp = False
+
+    def __enter__(self):
+        if self._parsed_path.scheme == "tcp":
+            self.is_tcp = True
+            self.open_socket()
+            return self
+        self._io_stream = open(self._path, "rb")  # pylint: disable=consider-using-with
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        self._io_stream.close()
+
+    def read(self, size: int = 4096) -> bytes:
+        """Read `size` bytes from file or socket.
+
+        :param size: of binary value to be read
+        :raise EOFError: if no value can be read or if it is empty.
+        :return: Read binary value
+        """
+        if self.is_tcp:
+            return self._io_stream.recv(size)
+
+        return self._io_stream.read(size)
+
+    def open_socket(self):
+        """Open tcp socket on provided path.
+        Tries to connect again if ConnectionRefusedError is raised"""
         tcp_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         while True:
             try:
-                tcp_socket.connect((parsed.hostname, parsed.port or 11120))
+                tcp_socket.connect(
+                    (self._parsed_path.hostname, self._parsed_path.port or 11120)
+                )
                 break
             except ConnectionRefusedError:
+                time.sleep(0.1)
                 continue
-        return tcp_socket
-
-    try:
-        file = open(path, "rb")  # pylint: disable=consider-using-with
-        return file
-    except Exception:  # pylint: disable=broad-except
-        print(f"Cannot open file:{path}\n")
-        return None
+        self._io_stream = tcp_socket
 
 
 _Tlv = namedtuple("TLV", "type length value")
 
 
-def iter_file(opened_file: BinaryIO) -> Iterator[_Tlv]:
+def iter_file(osef_stream: OsefStream, auto_reconnect: bool = True) -> Iterator[_Tlv]:
     """Iterator function to iterate over each frame in osef file.
 
-    :param opened_file: opened binary file containing tlv frames.
+    :param osef_stream: opened binary file containing tlv frames.
+    :param auto_reconnect: enable reconnection for tcp connections.
     :return frame_tlv: next tlv frame of the osef file.
     """
     while True:
         try:
-            frame_tlv = _read_next_tlv(opened_file)
+            frame_tlv = _read_next_tlv(osef_stream)
+        except EOFError:
+            if auto_reconnect and osef_stream.is_tcp:
+                logging.warning("connection lost: reopening socket")
+                osef_stream.open_socket()
+                continue
+            break
         except Exception:  # pylint: disable=broad-except
-            print(
+            logging.error(
                 "Error: cannot read next Tlv from file (malformated Tlv?).\n"
                 + f"Details: {traceback.format_exc()}"
                 + "\n"
             )
             break
-        if frame_tlv is None:
-            break
 
         yield frame_tlv
 
 
 def get_tlv_iterator(
-    opened_file: BinaryIO, first: int = None, last: int = None
+    opened_file: OsefStream,
+    first: int = None,
+    last: int = None,
+    auto_reconnect: bool = True,
 ) -> Iterable[_Tlv]:
     """Get an iterator to iterate over each tlv frame in osef file.
 
     :param opened_file: opened binary file containing tlv frames.
     :param first: iterate only on N first frames of file.
     :param last: iterate only on M last frames of file.
-    Can be used with first to get the range (N-M)..N
+    Can be used with first to get the range (N-M) -> N
+    :param auto_reconnect: enable reconnection for tcp connections.
     :return: tlv frame iterator
     """
     if first is None and last is None:
-        return enumerate(iter_file(opened_file))
-    return deque(islice(enumerate(iter_file(opened_file)), first), last)
+        return enumerate(iter_file(opened_file, auto_reconnect))
+    return deque(islice(enumerate(iter_file(opened_file, auto_reconnect)), first), last)
 
 
 _TreeNode = namedtuple("TreeNode", "type children leaf_value")
@@ -136,25 +168,26 @@ def parse_to_dict(frame_tree: _TreeNode) -> dict:
 
 
 def parse(
-    path: str, first: Optional[int] = None, last: Optional[int] = None
+    path: str,
+    first: Optional[int] = None,
+    last: Optional[int] = None,
+    auto_reconnect: bool = True,
 ) -> Iterator[dict]:
     """Iterator that opens and convert each tlv frame to a dict.
 
     :param path: path to osef file or TCP stream if path has form *tcp://hostname:port*
     :param first: iterate only on N first frames of file.
     :param last: iterate only on M last frames of file.
-    Can be used with first to get the range (N-M)..N
+    Can be used with first to get the range (N-M) ... N
+    :param auto_reconnect: enable reconnection for tcp connections.
     :return: next tlv dictionary
     """
-    osef_io = open_osef(path)
-    if not osef_io:
-        raise FileNotFoundError(f"Can't open file/stream from provided path: {path}")
-    iterator = get_tlv_iterator(osef_io, first, last)
-    for _, tlv in iterator:
-        raw_tree = build_tree(tlv)
-        if raw_tree:
-            yield parse_to_dict(raw_tree)
-    osef_io.close()
+    with OsefStream(path) as osef_stream:
+        iterator = get_tlv_iterator(osef_stream, first, last, auto_reconnect)
+        for _, tlv in iterator:
+            raw_tree = build_tree(tlv)
+            if raw_tree:
+                yield parse_to_dict(raw_tree)
 
 
 # -- Tlv Parsing --
@@ -174,40 +207,30 @@ def _align_size(size: int) -> int:
     return size if offset == 0 else size + alignment_size - offset
 
 
-# Add read method to socket to get same API as file
-setattr(socket.socket, "read", lambda self, size=4096: self.recv(size))
-
-
-def _read_from_file(osef_io: BinaryIO, byte_number: int) -> bytes:
+def _read_from_file(osef_stream: OsefStream, byte_number: int) -> bytes:
     """Read given number of bytes from readable stream"""
     blob = b""
     while len(blob) < byte_number:
-        blob_inc = osef_io.read(byte_number - len(blob))
+        blob_inc = osef_stream.read(byte_number - len(blob))
+        blob += blob_inc
 
         # End of file
         if blob_inc is None or len(blob_inc) == 0:
             raise EOFError
-
-        blob += blob_inc
-
     return blob
 
 
-def _read_next_tlv(osef_io: BinaryIO) -> Optional[_Tlv]:
+def _read_next_tlv(osef_stream: OsefStream) -> Optional[_Tlv]:
     """Read the next TLV from a binary stream (file or socket)"""
     # Read header
     struct = Struct(_STRUCT_FORMAT % 0)
-    try:
-        blob = _read_from_file(osef_io, struct.size)
-    except EOFError:
-        return None
-
+    blob = _read_from_file(osef_stream, struct.size)
     # Parse Type and Length
     read_tlv = _Tlv._make(struct.unpack_from(blob))
 
     # Now that we know its length we can read the Value
     struct = Struct(_STRUCT_FORMAT % read_tlv.length)
-    blob += _read_from_file(osef_io, struct.size - len(blob))
+    blob += _read_from_file(osef_stream, struct.size - len(blob))
     read_tlv = _Tlv._make(struct.unpack_from(blob))
 
     return read_tlv
@@ -240,7 +263,7 @@ def _parse_raw_to_tuple(raw_tree: _TreeNode) -> Tuple[str, Any]:
     if isinstance(type_info.node_info, types.LeafInfo):
         return type_info.name, unpack_value(leaf_value, type_info.node_info)
 
-    # For non-leaves, add each children to a dictionary
+    # For non-leaves, add each child to a dictionary
     tree = {}
     children_list = []
     for child in children:
