@@ -169,10 +169,19 @@ def _class_array_parser(value: bytes) -> np.ndarray:
 
 
 def _pose_parser(value: bytes) -> dict:
+    """Values to parse: tx ty tz Vxx Vyx Vzx Vxy Vyy Vzy Vxz Vyz Vzz
+
+    Where rotation matrices should be at the end:
+        | Vxx Vxy Vxz |
+    R = | Vyx Vyy Vyz |
+        | Vzx Vzy Vzz |
+    """
     floats = Struct("<ffffffffffff").unpack(value)
+    # we have to transpose rotation matrices because values
+    # are received column by column and not line by line
     return {
         "translation": np.array(floats[0:3]),
-        "rotation": np.reshape(np.array(floats[3:]), (3, 3)),
+        "rotation": np.transpose(np.reshape(np.array(floats[3:]), (3, 3))),
     }
 
 
